/*
 *  =======================================================================
 *  BoundingBox.java: Compute and store coordinates of an axis-aligned
 *  bounding box.
 *
 *  Written By: Mark Austin                                     April, 2019
 *  =======================================================================
 */

package geometry;

import java.lang.Math;

public class BoundingBox {
   private double dMinX, dMinY;
   private double dMaxX, dMaxY;
   private boolean DEBUG = false;

   // Constructor to create a null envelope ...

   public BoundingBox() {
      init();
   }

   public BoundingBox( double dX, double dY ) {
      init();
      addPoint( dX, dY );
   }

   // Create an envelope for a region defined by maximum and minimum values.

   public BoundingBox(double x1, double x2, double y1, double y2) {
      setToNull();
      addPoint( Math.min( x1, x2), Math.min( y1, y2) );
      addPoint( Math.max( x1, x2), Math.max( y1, y2) );
   }

   public BoundingBox( Node2D n1, Node2D n2 ) {
      setToNull();
      addPoint( Math.min( n1.getX(), n2.getX()), Math.min( n1.getY(), n2.getY() ) );
      addPoint( Math.max( n1.getX(), n2.getX()), Math.max( n1.getY(), n2.getY() ) );
   }

   // ==================================================================
   // Initialize the bounding box ....
   // ==================================================================

   public void init() {
      setToNull();
   }

   // ==================================================================
   // Make a "null" envelope containing an empty geometry ...
   // ==================================================================

   public void setToNull() {
      dMinX =  0;
      dMaxX = -1;
      dMinY =  0;
      dMaxY = -1;
   }

   // ==================================================================
   // Simple coordinate test to detect a null envelope ...
   // ==================================================================

   public boolean isNull() {
      return dMaxX < dMinX;
   }

   // ==================================================================
   // Expand boundary envelope so that it contains a new point (x,y) ...
   // ==================================================================

   public void addPoint(double x, double y) {
      if (isNull() == true ) {
         dMinX = dMaxX = x;
         dMinY = dMaxY = y;
      } else {
         dMinX = Math.min ( dMinX, x );
         dMaxX = Math.max ( dMaxX, x );
         dMinY = Math.min ( dMinY, y );
         dMaxY = Math.max ( dMaxY, y );
      }
   }

   // ===========================================
   // Compute and retrieve the envelope width ...
   // ===========================================

   public double getWidth() {
      if ( isNull() == true ) {
         return 0;
      }

      return dMaxX - dMinX;
   }

   // ===========================================
   // Compute and retrieve the envelope height ...
   // ===========================================

   public double getHeight() {
      if (isNull() == true ) {
         return 0;
      }

      return dMaxY - dMinY;
   }

   // ========================================================================
   // Retrieve x and y values of corner points ...
   // ========================================================================

   public double getMinX() {
      return dMinX;
   }

   public double getMaxX() {
      return dMaxX;
   }

   public double getMinY() {
      return dMinY;
   }

   public double getMaxY() {
      return dMaxY;
   }

   // ========================================================================
   // Simple test for point containment ....
   // ========================================================================

   public boolean contains( Node2D pt ) {
      boolean insideX = ( pt.getX() > dMinX ) && ( pt.getX() < dMaxX );
      boolean insideY = ( pt.getY() > dMinY ) && ( pt.getY() < dMaxY );
      return ( insideX && insideY );
   }

   // ========================================================================
   // Simple check if bounding boxes intersect ...
   // ========================================================================

   public boolean intersect( BoundingBox box ) {

      if( DEBUG == true) {
         System.out.println("--- this.getMinX()  --> " + this.getMinX() );
         System.out.println("--- this.getMaxX()  --> " + this.getMaxX() );
         System.out.println("--- this.getMinY()  --> " + this.getMinY() );
         System.out.println("--- this.getMaxY()  --> " + this.getMaxY() );
         System.out.println("--- box.getMinX()   --> " + box.getMinX() );
         System.out.println("--- box.getMaxX()   --> " + box.getMaxX() );
         System.out.println("--- box.getMinY()   --> " + box.getMinY() );
         System.out.println("--- box.getMaxY()   --> " + box.getMaxY() );
      }

      boolean part01 = this.getMinX() <= box.getMaxX();
      boolean part02 = this.getMaxX() >= box.getMinX();
      boolean part03 = this.getMinY() <= box.getMaxY();
      boolean part04 = this.getMaxY() >= box.getMinY();

      if( DEBUG == true) {
         System.out.println("--- this.getMinX() <= box.getMaxX() --> " + part01 );
         System.out.println("--- this.getMaxX() >= box.getMinX() --> " + part02 );
         System.out.println("--- this.getMinY() <= box.getMaxY() --> " + part03 );
         System.out.println("--- this.getMaxY() >= box.getMinY() --> " + part04 );
      }

      boolean result = part01 && part02 && part03 && part04;
      return result;
   }

   // ========================================================================
   // String representation for rectangular envelope ...
   // ========================================================================

   public String toString() {
      StringBuffer buf = new StringBuffer();

      buf.append(String.format("boundingbox[ x = (%f, %f), y = (%f, %f) ]", dMinX, dMaxX, dMinY, dMaxY ));

      return buf.toString();
   }
}
