/* 
 *  ========================================================================
 *  DemoSeattleWeather02.java: Create time series plot by reading data from
 *                             seattle-weather.csv file.
 * 
 *  Written by: Mark Austin                                    November 2021 
 *  ========================================================================
 */ 

package jfreechart;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;

import java.util.List;
import java.util.Date;

import java.time.LocalDate;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.ui.RectangleInsets;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.data.time.Day;
import org.jfree.data.time.Month;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;

public class DemoSeattleWeather02 extends JFrame {

    // Constructor meethods ...

    public DemoSeattleWeather02() {
       try {
          initUI();
       } catch(IOException e) {}
    }

    public DemoSeattleWeather02( String name ) {
       super (name);
       try {
          initUI();
       } catch(IOException e) {}

    }

    // Initialize datasets ...

    private void initUI() throws IOException {
        XYDataset dataset = createDataset();
        JFreeChart chart  = createChart(dataset);

        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
        chartPanel.setBackground(Color.white);
        
        add(chartPanel);

        pack();
        setLocationRelativeTo(null);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    // ================================================================
    // Create dataset from data/stocks.csv
    // ================================================================

    private XYDataset createDataset() throws IOException {

        // Create placeholders for stock time series ....

        TimeSeries maxtemp = new TimeSeries("Max Temp");
        TimeSeries mintemp = new TimeSeries("Min Temp");

        // Open CSVReader and skip over header ...

        CSVReader reader = new CSVReader( new FileReader("data/seattle-weather.csv") );
        String header [] = reader.readNext();

        // Loop over individual weather indicies ...

        String [] nextLine;
        while ((nextLine = reader.readNext()) != null) {
            String date           = nextLine[0];
            Double precipitation  = Double.parseDouble( nextLine[1] );
            Double maxTemperature = Double.parseDouble( nextLine[2] );
            Double minTemperature = Double.parseDouble( nextLine[3] );

            // Create date object from string input ...

            LocalDate dateItem = LocalDate.parse( date );
            int day   = dateItem.getDayOfMonth();
            int month = dateItem.getMonthValue();
            int year  = dateItem.getYear();

            maxtemp.add( new Day(day, month, year), maxTemperature );
            mintemp.add( new Day(day, month, year), minTemperature );
        }

        // Assemble collection of x-y lines ...

        TimeSeriesCollection dataset = new TimeSeriesCollection();
        dataset.addSeries(maxtemp);
        dataset.addSeries(mintemp);

        return dataset;
    }

    private JFreeChart createChart(final XYDataset dataset) {

        JFreeChart chart = ChartFactory.createTimeSeriesChart(
            "Max/Min Daily Temperature (C)", // title
            "Date",                          // x-axis label
            "Temperature (C)",               // y-axis label
            dataset,               // data
            true,                  // create legend?
            true,                  // generate tooltips?
            false                  // generate URLs?
        );

        chart.setBackgroundPaint(Color.white);

        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.white);
        plot.setDomainGridlinePaint(Color.black);
        plot.setRangeGridlinePaint(Color.black);
        plot.setAxisOffset( new RectangleInsets(5.0, 5.0, 5.0, 5.0) );
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
       
        XYItemRenderer r = plot.getRenderer();
        if (r instanceof XYLineAndShapeRenderer) {
            XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
            renderer.setSeriesPaint(0, Color.RED);
            renderer.setSeriesStroke(0, new BasicStroke(2.0f));
            renderer.setSeriesPaint(1, Color.BLUE);
            renderer.setSeriesStroke(1, new BasicStroke(2.0f));
        }

        DateAxis axis = (DateAxis) plot.getDomainAxis();
        axis.setDateFormatOverride(new SimpleDateFormat("MMM-yyyy"));

        chart.getLegend().setFrame(BlockBorder.NONE);
        chart.setTitle(new TextTitle("Max/Min Daily Temperature (C) in Seattle (Jan 1, 2012 -- Dec 31, 2015)",
                       new Font("Serif", Font.BOLD, 18)));

        return chart;
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(() -> {
            DemoSeattleWeather02 ex = new DemoSeattleWeather02("Max/Min Daily Temperature in Seattle");
            ex.setVisible(true);
        });
    }
}
